#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckGetKBOnServices(FileContentPlugin):
    name = "check_get_kb_on_services"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """
        Checks a given file if it is accessing one or more "Services/" KB keys
        like e.g.

        get_kb_item("Services/www");
        get_kb_list("Services/udp/upnp");

        These calls should use a "wrapping" function like e.g. the following
        (depending on the Service KB key) instead:

        http_get_port()
        service_get_port()
        service_get_ports()

        Args:
                nasl_file: The VT that is going to be checked
                file_content: The content of the file that is going to be
                              checked

        """
        if nasl_file.suffix == ".inc":
            return

        kb_matches = re.finditer(
            r'(get_kb_(?P<type>item|list)\s*\(\s*(?P<quote>[\'"])(?P<value>'
            r"Services/[^)]+)\))",
            file_content,
        )
        if kb_matches:
            for kb_match in kb_matches:
                if kb_match:
                    # special cases where not function currently exists
                    if "Services/tcp/*" in kb_match.group(
                        "value"
                    ) or "Services/udp/*" in kb_match.group("value"):
                        continue

                    # another special case, the find_service*.nasl need to
                    # access "Services/unknown" directly.
                    # The same is valid for unknown_services.nasl as well.
                    if "unknown_services.nasl" in str(nasl_file) or re.search(
                        r"find_service([0-9]+|_("
                        r"3digits|spontaneous|nmap|nmap_wrapped))?\.nasl",
                        str(nasl_file),
                    ):
                        continue

                    # an additional special case, this needs to access the
                    # KB key directly
                    if "2017/gb_hp_printer_rce_vuln.nasl" in str(nasl_file):
                        continue

                    yield LinterError(
                        f"The following get_kb_{kb_match.group('type')}() call"
                        " should use a function instead of a direct access to "
                        f"the Services/KB key: {kb_match.group('value')}",
                        file=self.context.nasl_file,
                        plugin=self.name,
                    )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckNewlines(FilePlugin):
    name = "check_wrong_newlines"

    def run(self) -> Iterator[LinterResult]:
        """This script FIXES newline errors:
        - Checking the passed VT for the existence of newlines in
          the script_name() and script_copyright() tags.
        - Search for (\r or \r\n).
        - Search for whitespaces in script_name( "myname") or script_copyright
        """
        # Need to be loaded as bytes or \r is converted to \n
        data = self.context.nasl_file.read_bytes()
        if b"\r" in data or b"\r\n" in data:
            yield LinterError("Found \\r or \\r\\n newline.")

        file_content = self.context.file_content

        # A few remaining have script_name( "myname") instead of
        # script_name("myname").
        # NEW: Remove whitespaces and newlines in script_name, script_copyright
        for tag in ["name", "copyright"]:
            whitespaces_match = re.search(
                rf'script_{tag}(?P<w1>\s*)\((?P<w2>\s*)(?P<quote>[\'"])'
                r"?.+?(?P=quote)?(?P<w3>\s*)\)(?P<w4>\s*);",
                file_content,
            )
            if whitespaces_match:
                for i in range(1, 5):
                    if whitespaces_match.group(f"w{i}") != "":
                        yield LinterError(
                            f"Found whitespaces in script_{tag}.",
                            file=self.context.nasl_file,
                            plugin=self.name,
                        )
                        break

            newline_match = re.search(
                rf'(script_{tag}\((?P<quote>[\'"])[^\'"\n;]*)[\n]+\s*'
                r'([^\'"\n;]*(?P=quote)\);)',
                file_content,
            )
            if newline_match:
                yield LinterError(
                    f"Found a newline within the tag script_{tag}.",
                    file=self.context.nasl_file,
                    plugin=self.name,
                )



# Copyright (C) 2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from pathlib import Path

from troubadix.helper import is_ignore_file
from troubadix.helper.helper import (
    get_path_from_root,
    get_root,
    is_enterprise_folder,
)


class IgnoreFile(unittest.TestCase):
    def test_compare_path_to_str(self):
        test_path = Path("foo/bar")

        self.assertTrue(is_ignore_file(test_path, ["foo"]))
        self.assertFalse(is_ignore_file(test_path, ["ipsum"]))

    def test_compare_path_to_path(self):
        test_path = Path("foo/bar")

        self.assertTrue(is_ignore_file(test_path, [Path("foo")]))
        self.assertFalse(is_ignore_file(test_path, [Path("ipsum")]))

    def test_compare_str_to_str(self):
        self.assertTrue(is_ignore_file("foo/bar", ["foo"]))
        self.assertFalse(is_ignore_file("foo/bar", ["ipsum"]))

    def test_compare_str_to_path(self):
        self.assertTrue(is_ignore_file("foo/bar", [Path("foo")]))
        self.assertFalse(is_ignore_file("foo/bar", [Path("ipsum")]))


class IsEnterpriseFolderTestCase(unittest.TestCase):
    def test_enterprise_folder(self):
        self.assertTrue(is_enterprise_folder(Path("enterprise")))
        self.assertTrue(is_enterprise_folder(Path("gsf")))
        self.assertTrue(is_enterprise_folder("enterprise"))
        self.assertTrue(is_enterprise_folder("gsf"))

        self.assertFalse(is_enterprise_folder("gcf"))
        self.assertFalse(is_enterprise_folder("foo"))
        self.assertFalse(is_enterprise_folder(Path("gcf")))
        self.assertFalse(is_enterprise_folder(Path("foo")))


class GetRootTestCase(unittest.TestCase):
    def test_get_root(self):
        self.assertEqual(get_root(Path("/nasl/foo/bar")), Path("/nasl"))
        self.assertEqual(get_root(Path("/nasl/common/bar")), Path("/nasl"))
        self.assertEqual(get_root(Path("/nasl/21.04/bar")), Path("/nasl"))
        self.assertEqual(get_root(Path("/nasl/22.04/bar")), Path("/nasl"))
        self.assertEqual(get_root(Path("/foo/bar")), Path("/"))
        self.assertEqual(get_root(Path("")), Path("/"))


class GetPathFromRootTestCase(unittest.TestCase):
    def test_get_path_from_root_relative(self):
        self.assertEqual(
            get_path_from_root(Path("nasl/foo"), Path("nasl/foo")), Path(".")
        )
        self.assertEqual(
            get_path_from_root(Path("nasl/foo"), Path("nasl")), Path("foo")
        )
        self.assertEqual(
            get_path_from_root(Path("nasl/foo/bar"), Path("nasl")),
            Path("foo/bar"),
        )

    def test_get_path_from_root_absolute(self):
        self.assertEqual(
            get_path_from_root(Path("/nasl/foo"), Path("/nasl/foo")), Path(".")
        )
        self.assertEqual(
            get_path_from_root(Path("/nasl/foo"), Path("/nasl")), Path("foo")
        )
        self.assertEqual(
            get_path_from_root(Path("/nasl/foo/bar"), Path("/nasl")),
            Path("foo/bar"),
        )

    # pylint: disable=expression-not-assigned
    def test_no_root_path(self):
        with self.assertRaises(ValueError):
            get_path_from_root(Path("nasl/foo/bar"), Path("nasl/baz/bar")),

        with self.assertRaises(ValueError):
            get_path_from_root(Path("/nasl/foo/bar"), Path("/nasl/baz/bar")),



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# pylint: disable=protected-access

import unittest
from pathlib import Path

from troubadix.results import FileResults
from troubadix.plugin import LinterWarning, LinterError


class TestResults(unittest.TestCase):
    def test_has_no_results(self):
        fresults = FileResults(file_path=Path("some/file.nasl"))
        fresults.add_plugin_results(plugin_name="test", results=list())

        self.assertFalse(fresults)

    def test_has_results(self):
        fresults = FileResults(file_path=Path("some/file.nasl"))
        fresults.add_plugin_results(
            plugin_name="test",
            results=[
                LinterError("test", file=Path("some/file.nasl"), plugin="test")
            ],
        )

        self.assertTrue(fresults)

    def test_has_results_ok_no_warning(self):
        fresults = FileResults(
            file_path=Path("some/file.nasl"), ignore_warnings=True
        )
        fresults.add_plugin_results(
            plugin_name="test",
            results=[
                LinterWarning(
                    "test", file=Path("some/file.nasl"), plugin="test"
                )
            ],
        )

        self.assertFalse(fresults)

    def test_has_results_no_warning(self):
        fresults = FileResults(
            file_path=Path("some/file.nasl"), ignore_warnings=True
        )
        fresults.add_plugin_results(
            plugin_name="test",
            results=[
                LinterWarning(
                    "test", file=Path("some/file.nasl"), plugin="test"
                )
            ],
        )
        fresults.add_plugin_results(
            plugin_name="test",
            results=[
                LinterError("test", file=Path("some/file.nasl"), plugin="test")
            ],
        )

        self.assertTrue(fresults)



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckRiskFactor(FilePlugin):
    name = "check_risk_factor"

    def run(self) -> Iterator[LinterResult]:
        """This script checks if a VT with risk_factor tag exist."""
        match = re.search(
            r'script_tag\(name:"risk_factor", value:"(?P<risk>.+)"\);',
            self.context.file_content,
        )

        if not match:
            return

        yield LinterError(
            f'Deprecated function "risk factor" found: {match.group("risk")}',
            file=self.context.nasl_file,
            plugin=self.name,
        )



# Copyright (c) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime, timedelta
from pathlib import Path
from unittest.mock import MagicMock

from troubadix.helper import CURRENT_ENCODING
from troubadix.helper.helper import get_path_from_root
from troubadix.plugin import LinterWarning
from troubadix.plugins.no_solution import CheckNoSolution

from . import PluginTestCase

here = Path(__file__).parent


class CheckNoSolutionTestCase(PluginTestCase):
    def test_ok(self):
        file1 = here / "test_files" / "nasl" / "21.04" / "test.nasl"
        text = file1.read_text(encoding=CURRENT_ENCODING)
        file1.write_text(
            text.replace(
                'name:"cvss_base", value:"0.0"', 'name:"cvss_base", value:"1.0"'
            ),
            encoding=CURRENT_ENCODING,
        )
        context = MagicMock()
        context.nasl_files = [file1]
        context.root = here
        plugin = CheckNoSolution(context)
        results = list(plugin.run())

        self.assertEqual(len(results), 0)
        file1.write_text(text, encoding=CURRENT_ENCODING)

    def test_ok_no_score(self):
        file1 = here / "test_files" / "nasl" / "21.04" / "test.nasl"
        context = MagicMock()
        context.nasl_files = [file1]
        context.root = here
        plugin = CheckNoSolution(context)
        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_solution_type(self):
        file1 = (
            here
            / "test_files"
            / "nasl"
            / "21.04"
            / "fail_solution_template.nasl"
        )
        text = file1.read_text(encoding=CURRENT_ENCODING)
        file1.write_text(
            text.replace("NoneAvailable", "WillNotFix"),
            encoding=CURRENT_ENCODING,
        )
        context = MagicMock()
        context.nasl_files = [file1]
        context.root = here
        plugin = CheckNoSolution(context)
        results = list(plugin.run())

        self.assertEqual(len(results), 0)
        file1.write_text(text, encoding=CURRENT_ENCODING)

    def test_ok_inc(self):
        file1 = here / "test_files" / "nasl" / "21.04" / "test.inc"

        context = MagicMock()
        context.nasl_files = [file1]
        context.root = here
        plugin = CheckNoSolution(context)
        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_too_old_1_year(self):
        file1 = (
            here
            / "test_files"
            / "nasl"
            / "21.04"
            / "fail_solution_template.nasl"
        )
        context = MagicMock()
        context.nasl_files = [file1]
        context.root = here
        plugin = CheckNoSolution(context)
        results = list(plugin.run())

        from_root = get_path_from_root(file1, context.root)

        self.assertEqual(len(results), 5)
        self.assertIsInstance(results[0], LinterWarning)
        self.assertIn(
            f"{from_root}: ",
            results[0].message,
        )
        self.assertIn(
            "Missing solution, older than 1 year.",
            results[0].message,
        )

    def test_too_old_6_months(self):
        file1 = (
            here
            / "test_files"
            / "nasl"
            / "21.04"
            / "fail_solution_template.nasl"
        )
        new_date = (datetime.now() - timedelta(days=200)).strftime("%Y/%m/%d")

        text = file1.read_text(encoding=CURRENT_ENCODING)
        file1.write_text(
            text.replace("02nd February, 2021", new_date),
            encoding=CURRENT_ENCODING,
        )

        context = MagicMock()
        context.nasl_files = [file1]
        context.root = here
        plugin = CheckNoSolution(context)
        results = list(plugin.run())

        # reverse change to file
        file1.write_text(text, encoding=CURRENT_ENCODING)
        from_root = get_path_from_root(file1, context.root)

        self.assertEqual(len(results), 5)
        self.assertIsInstance(results[0], LinterWarning)
        self.assertIn(
            f"{from_root}: ",
            results[0].message,
        )
        self.assertIn(
            "Missing solution, older than 6 months.",
            results[0].message,
        )

    def test_too_young_31_days(self):
        file1 = (
            here
            / "test_files"
            / "nasl"
            / "21.04"
            / "fail_solution_template.nasl"
        )
        new_date = (datetime.now() - timedelta(days=10)).strftime("%Y/%m/%d")

        text = file1.read_text(encoding=CURRENT_ENCODING)
        file1.write_text(
            text.replace("02nd February, 2021", new_date),
            encoding=CURRENT_ENCODING,
        )

        context = MagicMock()
        context.nasl_files = [file1]
        context.root = here
        plugin = CheckNoSolution(context)
        results = list(plugin.run())

        # reverse change to file
        file1.write_text(text, encoding=CURRENT_ENCODING)
        from_root = get_path_from_root(file1, context.root)

        self.assertEqual(len(results), 5)
        self.assertIsInstance(results[0], LinterWarning)
        self.assertIn(
            f"{from_root}: ",
            results[0].message,
        )
        self.assertIn(
            "Missing solution, but younger than 31 days.",
            results[0].message,
        )

    def test_multiples(self):
        file1 = (
            here
            / "test_files"
            / "nasl"
            / "21.04"
            / "fail_solution_template.nasl"
        )
        file2 = (
            here
            / "test_files"
            / "nasl"
            / "21.04"
            / "fail_solution_template2.nasl"
        )
        file3 = (
            here
            / "test_files"
            / "nasl"
            / "21.04"
            / "fail_solution_template3.nasl"
        )

        date1 = (datetime.now() - timedelta(days=200)).strftime("%Y/%m/%d")
        date2 = (datetime.now() - timedelta(days=400)).strftime("%Y/%m/%d")
        date3 = (datetime.now() - timedelta(days=10)).strftime("%Y/%m/%d")

        text = file1.read_text(encoding=CURRENT_ENCODING)

        file1.write_text(
            text.replace("02nd February, 2021", date1),
            encoding=CURRENT_ENCODING,
        )
        file2.write_text(
            text.replace("02nd February, 2021", date2),
            encoding=CURRENT_ENCODING,
        )
        file3.write_text(
            text.replace("02nd February, 2021", date3),
            encoding=CURRENT_ENCODING,
        )

        context = MagicMock()
        context.nasl_files = [file1, file2, file3]
        context.root = here
        plugin = CheckNoSolution(context)
        results = list(plugin.run())

        # reverse change to file
        file1.write_text(text, encoding=CURRENT_ENCODING)
        file2.unlink()
        file3.unlink()

        self.assertEqual(len(results), 7)
        self.assertEqual(
            "total missing solutions: 3",
            results[3].message,
        )
        self.assertEqual(
            "missing solutions younger 1 month: 1",
            results[4].message,
        )
        self.assertEqual(
            "missing solutions older than 6 months: 1",
            results[5].message,
        )
        self.assertEqual(
            "missing solutions older than 1 year: 1",
            results[6].message,
        )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest
import tempfile
from contextlib import contextmanager
from pathlib import Path
from typing import Generator
from subprocess import SubprocessError
from troubadix.standalone_plugins.changed_oid import check_oid, git, parse_args


@contextmanager
def tempgitdir() -> Generator[Path, None, None]:
    cwd = Path.cwd()
    tempdir = tempfile.TemporaryDirectory()
    temppath = Path(tempdir.name)
    os.chdir(str(temppath))
    git("init", "-b", "main")
    git("config", "--local", "user.email", "max.mustermann@example.com")
    git("config", "--local", "user.name", "Max Mustermann")
    yield temppath
    tempdir.cleanup()
    os.chdir(str(cwd))


def testgit(tmpdir: Path, ok: bool = False) -> None:
    test_file = tmpdir / "test.nasl"
    test_file.write_text('script_oid("1.3.6.1.4.1.25623.1.0.100313");')
    git("add", str(test_file))
    git("commit", "-m", "test")
    git("checkout", "-b", "test")
    if ok:
        test_file.write_text(
            'script_oid("1.3.6.1.4.1.25623.1.0.100313");\ntest'
        )
    else:
        test_file.write_text('script_oid("2.3.6.1.4.1.25623.1.0.100313");')
    git("add", "-u")
    git("commit", "-m", "test2")


class TestChangedOid(unittest.TestCase):
    def test_check_oid_ok(self):
        with tempgitdir() as tmpdir:
            testgit(tmpdir, True)
            parsed_args = parse_args(["-c", "main..test"])
            self.assertFalse(check_oid(parsed_args))

    def test_check_oid_fail(self):
        with tempgitdir() as tmpdir:
            testgit(tmpdir)
            parsed_args = parse_args(["-c", "main..test"])
            self.assertTrue(check_oid(parsed_args))

    def test_git_fail(self):
        with self.assertRaises(SubprocessError):
            git("bla")

    def test_git_ok(self):
        git("--version")

    def test_args_ok(self):
        with tempgitdir() as tmpdir:
            testgit(tmpdir)
            self.assertEqual(
                parse_args(["-c", "main..test"]).commit_range, "main..test"
            )
            self.assertEqual(
                parse_args(["-c", "main..test", "-f", "test.nasl"]).files,
                [Path("test.nasl")],
            )


if __name__ == "__main__":
    unittest.main()



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.forking_nasl_functions import CheckForkingNaslFunctions

from . import PluginTestCase


class CheckForkingNaslFunctionsTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'get_app_port_from_cpe_prefix("cpe:/o:foo:bar");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckForkingNaslFunctions(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckForkingNaslFunctions(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_not_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            "if( ! infos = get_app_port_from_cpe_prefix( cpe:CPE_PREFIX, "
            'service:"www" ) )\nexit( 0 );\n'
            "if( ! infos = get_app_port_from_cpe_prefix( cpe:CPE_PREFIX, "
            'service:"www" ) )\nexit( 0 );\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckForkingNaslFunctions(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "The VT is using the "
            'get_app_port_from_cpe_prefix( cpe:CPE_PREFIX, service:"www" ) ) '
            "multiple times or in conjunction with other forking functions. "
            "Please either use get_app_port_from_list() from host_details.inc "
            "or split your VT into several VTs for each covered protocol.",
            results[0].message,
        )

    def test_not_ok2(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            "if(!infos = get_app_full(cpe:CPE, port:port, exit_no_version:TRUE)"
            ")\nexit(0);\n"
            "if(!infos = get_app_full(cpe:CPE, port:port, exit_no_version:TRUE)"
            ")\nexit(0);\n"
        )

        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckForkingNaslFunctions(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "The VT is using the "
            "get_app_full(cpe:CPE, port:port, exit_no_version:TRUE)) "
            "multiple times or in conjunction with other forking functions. "
            "Please use e.g. get_app_version_and_location(), "
            "get_app_version_and_location_from_list() or similar functions "
            "from host_details.inc.",
            results[0].message,
        )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import (
    _get_special_script_tag_pattern,
    _get_tag_pattern,
)
from troubadix.plugin import FileContentPlugin, LinterResult, LinterWarning


class CheckScriptCallsRecommended(FileContentPlugin):
    name = "check_script_calls_recommended"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """
        This script checks for the existence of recommended script calls. These
        are categorize int two groups. In group 2 it is recommended to call
        every single one. In group 1 it is sufficient to call one of the script
        calls.

        group1:
        - script_dependencies

        group2:
        - script_require_ports
        - script_require_udp_ports
        - script_require_keys
        - script_mandatory_keys
        """
        if nasl_file.suffix == ".inc":
            return

        if _get_special_script_tag_pattern(
            name=r"category", value=r"ACT_(SETTINGS|SCANNER|INIT)"
        ).search(file_content) or _get_tag_pattern(
            name=r"deprecated", value=r"TRUE"
        ).search(
            file_content
        ):
            return

        recommended_single_call = [r"dependencies"]
        recommended_many_call = [
            r"require_ports",
            r"require_udp_ports",
            r"require_keys",
            r"mandatory_keys",
        ]

        if not _get_special_script_tag_pattern(
            name=rf"({'|'.join(recommended_many_call)})", value=".*"
        ).search(file_content):
            yield LinterWarning(
                "VT contains none of the following recommended calls: "
                f"{', '.join(recommended_many_call)}",
                file=nasl_file,
                plugin=self.name,
            )
        for call in recommended_single_call:
            if not _get_special_script_tag_pattern(
                name=call, value=".*", flags=re.DOTALL
            ).search(file_content):
                yield LinterWarning(
                    "VT does not contain the following recommended call: "
                    f"'script_{call}'",
                    file=nasl_file,
                    plugin=self.name,
                )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.vt_file_permission import CheckVTFilePermissions

from . import PluginTestCase


class CheckVTFilePermissionsTestCase(PluginTestCase):
    def test_ok(self):
        fake_context = self.create_file_plugin_context(
            nasl_file=Path(__file__).parent
            / "test_files"
            / "ok_permissions.nasl"
        )

        plugin = CheckVTFilePermissions(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_nok(self):
        fake_context = self.create_file_plugin_context(
            nasl_file=Path(__file__).parent
            / "test_files"
            / "fail_permissions.nasl"
        )

        plugin = CheckVTFilePermissions(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            results[0].message,
            "VT has invalid file permissions: -rwxr-xr-x.\n"
            "NASL scripts must not be executable.\n"
            "Typical file permissions are '644' (-rw-r--r-) "
            "and `664` (-rw-rw-r-)",
        )



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckDescription(FilePlugin):
    name = "check_description"

    def run(self) -> Iterator[LinterResult]:

        """This script checks if some NVTs are still using script_description

        Args:
            nasl_file:    The VT / Include that is going to be checked
            file_content: The content of the file that is going to be
                          checked
        """
        script_description = re.compile(
            r"script_description", re.IGNORECASE
        ).search(self.context.file_content)
        if script_description:
            yield LinterError(
                "VT/Include is using deprecated 'script_description'",
                file=self.context.nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.vt_placement import CheckVTPlacement

from . import PluginTestCase


class CheckVTPlacementTestCase(PluginTestCase):
    def test_ok(self):
        with self.create_directory() as tempdir:
            path = tempdir / "file.nasl"
            for _type in ["Product", "Service"]:
                content = (
                    'script_tag(name:"cvss_base", value:"4.0");\n'
                    'script_tag(name:"summary", value:"Foo Bar.");\n'
                    f'script_family("{_type} detection");\n'
                )
                fake_context = self.create_file_plugin_context(
                    nasl_file=path, file_content=content, root=path.parent
                )
                plugin = CheckVTPlacement(fake_context)

                results = list(plugin.run())

                self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckVTPlacement(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok_dirs(self):
        with self.create_directory() as tempdir:
            for _dir in ["gsf", "attic"]:
                path = tempdir / _dir / "file.nasl"
                for _type in ["Product", "Service"]:
                    content = (
                        'script_tag(name:"cvss_base", value:"4.0");\n'
                        'script_tag(name:"summary", value:"Foo Bar.");\n'
                        f'script_family("{_type} detection");\n'
                    )
                    fake_context = self.create_file_plugin_context(
                        nasl_file=path, file_content=content, root=tempdir
                    )
                    plugin = CheckVTPlacement(fake_context)

                    results = list(plugin.run())

                    self.assertEqual(len(results), 0)

    def test_ok_deprecated(self):
        with self.create_directory() as tempdir:
            path = tempdir / "file.nasl"
            for _type in ["Product", "Service"]:
                content = (
                    'script_tag(name:"cvss_base", value:"4.0");\n'
                    'script_tag(name:"summary", value:"Foo Bar.");\n'
                    f'script_family("{_type} detection");\n'
                    'script_tag(name:"deprecated", value=TRUE);\n'
                )

                fake_context = self.create_file_plugin_context(
                    nasl_file=path, file_content=content, root=tempdir
                )
                plugin = CheckVTPlacement(fake_context)

                results = list(plugin.run())

                self.assertEqual(len(results), 0)

    def test_no_detection(self):
        with self.create_directory() as tempdir:
            path = tempdir / "file.nasl"
            content = (
                'script_tag(name:"cvss_base", value:"4.0");\n'
                'script_tag(name:"summary", value:"Foo Bar...");\n'
                'script_dependencies("example.inc");\n'
            )

            fake_context = self.create_file_plugin_context(
                nasl_file=path, file_content=content, root=tempdir
            )
            plugin = CheckVTPlacement(fake_context)

            results = list(plugin.run())

            self.assertEqual(len(results), 0)

    def test_wrong_placement(self):
        with self.create_directory() as tempdir:
            path = tempdir / "foo" / "bar" / "file.nasl"
            for _type in ["Product", "Service"]:
                content = (
                    'script_tag(name:"cvss_base", value:"4.0");\n'
                    'script_tag(name:"summary", value:"Foo Bar.");\n'
                    f'script_family("{_type} detection");\n'
                )
                fake_context = self.create_file_plugin_context(
                    nasl_file=path, file_content=content, root=tempdir
                )
                plugin = CheckVTPlacement(fake_context)

                results = list(plugin.run())

                self.assertEqual(len(results), 1)
                self.assertIsInstance(results[0], LinterError)
                self.assertEqual(
                    "VT should be "
                    f"placed in the root directory ({tempdir}).",
                    results[0].message,
                )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.helper import CURRENT_ENCODING
from troubadix.plugin import LinterError
from troubadix.plugins.newlines import CheckNewlines

from . import PluginTestCase


class CheckNewlinesTestCase(PluginTestCase):
    def test_ok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckNewlines(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_newline_in_name(self):
        nasl_file = (
            Path(__file__).parent / "test_files" / "fail_name_newline.nasl"
        )
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckNewlines(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Found a newline within the tag script_name.",
            results[0].message,
        )

    def test_newline_in_name_and_copyright(self):
        nasl_file = (
            Path(__file__).parent
            / "test_files"
            / "fail_name_and_copyright_newline.nasl"
        )
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckNewlines(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Found a newline within the tag script_name.",
            results[0].message,
        )
        self.assertIsInstance(results[1], LinterError)
        self.assertEqual(
            "Found a newline within the tag script_copyright.",
            results[1].message,
        )

    def test_whitespaces_in_name_and_copyright(self):
        nasl_file = (
            Path(__file__).parent
            / "test_files"
            / "fail_name_and_copyright_newline.nasl"
        )
        content = (
            'script_name( "foodetection");\n'
            'script_copyright ( "Copyright(c) Greenbone Networks GmbH" ) ; \n'
            'script_copyright ("Copyright(c) Greenbone Networks GmbH");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckNewlines(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Found whitespaces in script_name.",
            results[0].message,
        )
        self.assertIsInstance(results[1], LinterError)
        self.assertEqual(
            "Found whitespaces in script_copyright.",
            results[1].message,
        )

    def test_new_line(self):
        nasl_file = (
            Path(__file__).parent / "test_files" / "fail_bad_new_line.nasl"
        )
        content = (
            'script_name("foo detection");'
            'script_copyright("Copyright(c) Greenbone Networks GmbH");\r\n'
            'script_copyright("Copyrigh(c) Greenbone Networks GmbH");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckNewlines(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Found \\r or \\r\\n newline.",
            results[0].message,
        )
        self.assertIsInstance(results[0], LinterError)



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.tabs import CheckTabs

from . import PluginTestCase


class CheckTabsTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("tests/file.nasl")
        content = "What ever."
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content.splitlines()
        )
        plugin = CheckTabs(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_with_tabs(self):
        path = Path("tests/file.nasl")

        content = "\t\t\t\t\t\t\n1234456789"

        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content.splitlines()
        )
        plugin = CheckTabs(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            results[0].message,
            "VT uses tabs instead of spaces.",
        )
        self.assertEqual(results[0].line, 1)



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from datetime import datetime, timedelta
from typing import Iterator

from troubadix.helper import CURRENT_ENCODING, ScriptTag, get_script_tag_pattern
from troubadix.helper.helper import get_path_from_root
from troubadix.plugin import (
    FilesPlugin,
    LinterError,
    LinterResult,
    LinterWarning,
)

NO_SOLUTION_DATE_TOO_YOUNG = datetime.now() - timedelta(days=31)
NO_SOLUTION_DATE_TOO_OLDER_6_MONTH = datetime.now() - timedelta(days=186)
NO_SOLUTION_DATE_TOO_OLDER_1_YEAR = datetime.now() - timedelta(days=365)

# Add the solutions date's here
STRPTIMES = ["%d %B, %Y", "%d %b, %Y", "%Y/%m/%d"]


class CheckNoSolution(FilesPlugin):
    name = "check_no_solution"

    def run(self) -> Iterator[LinterResult]:
        """Run PRE_RUN_COLLECTOR."""

        total_missing_solutions = 0
        missing_solutions_younger_1_month = 0
        missing_solutions_older_than_6_months = 0
        missing_solutions_older_than_1_year = 0

        for nasl_file in self.context.nasl_files:
            if nasl_file.suffix == ".inc":
                continue

            content = nasl_file.read_text(encoding=CURRENT_ENCODING)

            st_pattern = get_script_tag_pattern(ScriptTag.SOLUTION_TYPE)
            st = st_pattern.search(content)
            if st and st.group("value") != "NoneAvailable":
                continue

            tag_pattern = get_script_tag_pattern(ScriptTag.CVSS_BASE)
            # don't need to check detection scripts since they don't refer
            # to CVEs. all detection scripts have a cvss of 0.0
            cvss_detect = tag_pattern.search(content)
            if cvss_detect and cvss_detect.group("value") == "0.0":
                continue

            solution_match = get_script_tag_pattern(ScriptTag.SOLUTION).search(
                content
            )

            if solution_match:
                date_match = re.compile(
                    r"as\s+of\s*(?P<date>.+?)\.\s*", re.DOTALL
                ).search(solution_match.group("value"))
            else:
                yield LinterError(
                    f"{get_path_from_root(nasl_file, self.context.root)}: "
                    "No Solution tag found.",
                    file=nasl_file,
                    plugin=self.name,
                )
                continue

            # total number of missing solutions
            if date_match:
                total_missing_solutions += 1
            else:
                continue

            no_solution_since = parse_date(date_match.group("date"))
            if not no_solution_since:
                yield LinterError(
                    f"{get_path_from_root(nasl_file, self.context.root)}: "
                    f"Can not convert '{date_match.group('date')}' to datetime",
                    file=nasl_file,
                    plugin=self.name,
                )
                continue

            # no solution and older than 1 year
            if no_solution_since <= NO_SOLUTION_DATE_TOO_OLDER_1_YEAR:
                missing_solutions_older_than_1_year += 1
                yield LinterWarning(
                    f"{get_path_from_root(nasl_file, self.context.root)}: "
                    "Missing solution, older than 1 year.",
                    file=nasl_file,
                    plugin=self.name,
                )
                continue

            # no solution and older than 6 months
            if no_solution_since <= NO_SOLUTION_DATE_TOO_OLDER_6_MONTH:
                missing_solutions_older_than_6_months += 1
                yield LinterWarning(
                    f"{get_path_from_root(nasl_file, self.context.root)}: "
                    "Missing solution, older than 6 months.",
                    file=nasl_file,
                    plugin=self.name,
                )
                continue

            # no solution and younger than 31 days
            if no_solution_since >= NO_SOLUTION_DATE_TOO_YOUNG:
                missing_solutions_younger_1_month += 1
                yield LinterWarning(
                    f"{get_path_from_root(nasl_file, self.context.root)}: "
                    "Missing solution, but younger than 31 days.",
                    file=nasl_file,
                    plugin=self.name,
                )

        if total_missing_solutions > 0:
            yield LinterWarning(
                "total missing solutions:" f" {total_missing_solutions}",
                plugin=self.name,
            )
            yield LinterWarning(
                "missing solutions younger 1 month:"
                f" {missing_solutions_younger_1_month}",
                plugin=self.name,
            )
            yield LinterWarning(
                "missing solutions older than 6 months:"
                f" {missing_solutions_older_than_6_months}",
                plugin=self.name,
            )
            yield LinterWarning(
                "missing solutions older than 1 year:"
                f" {missing_solutions_older_than_1_year}",
                plugin=self.name,
            )


def parse_date(date_string: str) -> datetime:
    """Convert date string to date trying different formats"""

    date_string = re.sub(
        r"(?P<date>.\d{1,2})(st|nd|rd|th)", r"\g<date>", date_string
    )

    for strptime in STRPTIMES:
        try:
            date = datetime.strptime(date_string, strptime)

            return date
        except ValueError:
            pass

    return None



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import re
from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import ScriptTag, get_script_tag_pattern
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckSecurityMessages(FileContentPlugin):
    name = "check_security_messages"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This script checks the passed VT if it is using a security_message
        and having no severity (CVSS score) assigned which is an error /
        debugging leftover in most cases.

        Args:
            nasl_file: The VT that is going to be checked
            file_content: The content of the VT

        """
        if nasl_file.suffix == ".inc":
            return

        # Policy VTs might use both, security_message and log_message
        if (
            "Policy/" in str(nasl_file)
            or "PCIDSS/" in str(nasl_file)
            or "GSHB/" in str(nasl_file)
        ):
            return

        # don't need to check VTs having a severity (which are for sure
        # using a security_message) or no cvss_base (which shouldn't happen and
        # is checked in a separate step) included at all.
        cvss_base_pattern = get_script_tag_pattern(ScriptTag.CVSS_BASE)
        cvss_detect = cvss_base_pattern.search(file_content)

        if cvss_detect and cvss_detect.group("value") != "0.0":
            return

        sec_match = re.search(
            r"security_message\s*\([\w:#.&\-!,<>\[\]("
            r")\s\"`+'/\\\n]+\)\s*(;|;\s*(\n|#))",
            file_content,
            re.MULTILINE,
        )

        if sec_match:
            yield LinterError(
                "VT is using a security_message in a VT without severity",
                file=nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# pylint: disable=protected-access

import io
import unittest
from contextlib import redirect_stdout
from pathlib import Path

from pontos.terminal.terminal import ConsoleTerminal

from troubadix.reporter import Reporter

_here = Path(__file__).parent


class TestReporter(unittest.TestCase):
    def setUp(self):
        self._term = ConsoleTerminal()
        self.root = _here / "plugins" / "test_files" / "nasl"

    def test_set_files_count(self):
        reporter = Reporter(root=self.root, term=self._term)

        reporter.set_files_count(count=1)

        self.assertEqual(1, reporter._files_count)

    def test_get_error_count(self):
        reporter = Reporter(root=self.root, term=self._term)

        self.assertEqual(reporter.get_error_count(), 0)

    def test_report_statistic(self):
        reporter = Reporter(root=self.root, term=self._term)

        with redirect_stdout(io.StringIO()) as f:
            reporter.report_statistic()

        output = f.getvalue()

        self.assertIn("-" * 67, output)

        self.assertIn("Errors Warnings", output)
        self.assertIn(f"{'sum':48} {0:8} {0:8}", output)

    def test_report_statistic_with_fix_and_ignore_warnings(self):
        reporter = Reporter(
            root=self.root, term=self._term, fix=True, ignore_warnings=True
        )

        with redirect_stdout(io.StringIO()) as f:
            reporter.report_statistic()

        output = f.getvalue()

        self.assertIn("-" * 67, output)

        self.assertIn("Errors    Fixes", output)
        self.assertIn(f"{'sum':48} {0:8} {0:8}", output)

    def test_report_statistic_with_fix(self):
        reporter = Reporter(root=self.root, term=self._term, fix=True)

        with redirect_stdout(io.StringIO()) as f:
            reporter.report_statistic()

        output = f.getvalue()

        self.assertIn("-" * 75, output)

        self.assertIn("Errors Warnings    Fixes", output)
        self.assertIn(f"{'sum':48} {0:8} {0:8} {0:8}", output)

    def test_report_statistic_with_ignore_warnings(self):
        reporter = Reporter(
            root=self.root, term=self._term, ignore_warnings=True
        )

        with redirect_stdout(io.StringIO()) as f:
            reporter.report_statistic()

        output = f.getvalue()

        self.assertIn("-" * 59, output)

        self.assertIn("Errors", output)
        self.assertIn(f"{'sum':48} {0:8}", output)

    def test_report_statistic_none(self):
        reporter = Reporter(root=self.root, term=self._term, statistic=False)

        with redirect_stdout(io.StringIO()) as f:
            reporter.report_statistic()

        output = f.getvalue()

        self.assertFalse(output)



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckForkingNaslFunctions(FileContentPlugin):
    name = "check_forking_nasl_functions"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This script checks if any of the following functions are used more
        than once or in conjunction with other mentioned functions within the
        same VT:

        - get_app_port();
        - get_app_port_from_list();
        - get_app_port_from_cpe_prefix();
        - service_get_port();
        - unknownservice_get_port();
        - ftp_get_port();
        - http_get_port();
        - telnet_get_port();
        - smtp_get_port();
        - pop3_get_port();
        - imap_get_port();
        - ssh_get_port();
        - tls_ssl_get_port();
        - ldap_get_port();
        - snmp_get_port();
        - sip_get_port_proto();
        - rsync_get_port();
        - nntp_get_port();
        - tcp_get_all_port();
        - udp_get_all_port();

        In addition, those specific functions *might* get a port from e.g.
        get_app_port() passed and are handled separately. These shouldn't be
        called together as well but the check is done independently of the
        ones above:

        - get_app_version();
        - get_app_location();
        - get_app_version_from_list();
        - get_app_version_and_location_from_list();
        - get_app_version_and_location();
        - get_app_location_and_proto();
        - get_app_version_and_proto();
        - get_app_full();
        - get_app_details();

        Args:
            nasl_file: The VT that is going to be checked
            file_content: The content of the file that is going to be checked
        """
        if nasl_file.suffix == ".inc":
            return

        # Those two are only calling http_get_port() if get_app_port() was
        # "empty".
        if "sw_magento_magmi_detect.nasl" in str(
            nasl_file
        ) or "2014/gb_apache_struts_classloader_vuln.nasl" in str(nasl_file):
            return

        # Those two are using if/else calls between
        # smtp_get_port/imap_get_port or get_app_port/service_get_port calls.
        if "2009/zabbix_37308.nasl" in str(
            nasl_file
        ) or "pre2008/mailenable_imap_rename_dos.nasl" in str(nasl_file):
            return

        # This one is using if/else calls similar to the examples above.
        if "2013/gb_sap_netweaver_portal_rce_04_13.nasl" in str(nasl_file):
            return

        # Another example using if/else calls
        if "gsf/2022/siemens/gb_sicam_a8000_ssa-316850.nasl" in str(nasl_file):
            return

        match = re.findall(
            r"\s*[=!]\s*((get_app_port|get_app_port_from_("
            r"cpe_prefix|list)|sip_get_port_proto|(tcp|udp)_get_all_port|("
            r"ftp|http|telnet|smtp|pop3|imap|ssh|tls_ssl|ldap|snmp|rsync|nntp"
            r"|unknownservice|service)_get_port)\s*\([^)]*\)\s*[;\)])",
            file_content,
        )
        if match and len(match) > 1:
            for tag in match:
                if tag[0]:
                    yield LinterError(
                        f"The VT is using the {tag[0]} "
                        "multiple times or in conjunction with other "
                        "forking functions. Please either use get_app_port_from"
                        "_list() from host_details.inc or split your VT into "
                        f"several VTs for each covered protocol.",
                        file=self.context.nasl_file,
                        plugin=self.name,
                    )
            return

        match = re.findall(
            r"\s*[=!]\s*(get_app_(version|location|version_from_list"
            r"|version_and_location_from_list|version_and_location"
            r"|location_and_proto|version_and_proto|full|details)\s*\(["
            r"^)]*\)\s*[;\)])",
            file_content,
        )
        if match and len(match) > 1:
            for tag in match:
                if tag[0]:
                    # some special cases, these are calling get_app_location
                    # with nofork:TRUE which returns a list instead of doing
                    # a fork.
                    exceptions = [
                        "2018/phpunit/gb_phpunit_rce.nasl",
                        "2018/gb_unprotected_web_app_installers.nasl",
                        "2018/gb_sensitive_file_disclosures_http.nasl",
                    ]
                    if any(e in str(nasl_file) for e in exceptions):
                        if "nofork:TRUE" in tag[0]:
                            continue

                    yield LinterError(
                        f"The VT is using the {tag[0]} multiple times or in "
                        "conjunction with other forking functions. Please use "
                        "e.g. get_app_version_and_location(), "
                        "get_app_version_and_location_from_list() or similar "
                        "functions from host_details.inc.",
                        file=self.context.nasl_file,
                        plugin=self.name,
                    )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError, LinterWarning
from troubadix.plugins.dependencies import CheckDependencies

from . import PluginTestCase

here = Path(__file__).parent


class CheckDependenciesTestCase(PluginTestCase):
    def test_ok(self):
        path = here / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=here
        )
        plugin = CheckDependencies(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = here / "file.inc"
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckDependencies(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_dependency_existing(self):
        with self.create_directory() as tmpdir:
            path = tmpdir / "file.nasl"
            example = tmpdir / "common" / "example.inc"
            example.parent.mkdir(parents=True)
            example.touch()
            content = (
                'script_tag(name:"cvss_base", value:"4.0");\n'
                'script_tag(name:"summary", value:"Foo Bar...");\n'
                'script_dependencies("example.inc");\n'
            )
            fake_context = self.create_file_plugin_context(
                nasl_file=path, file_content=content, root=tmpdir
            )
            plugin = CheckDependencies(fake_context)

            results = list(plugin.run())

            self.assertEqual(len(results), 0)

    def test_dependency_missing(self):
        dependency = "example2.inc"
        path = here / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar...");\n'
            f'script_dependencies("{dependency}");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=here
        )
        plugin = CheckDependencies(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            f"The script dependency {dependency} could "
            "not be found within the VTs.",
            results[0].message,
        )

    def test_enterprise_dependency(self):
        with self.create_directory() as tmpdir:
            path = tmpdir / "file.nasl"
            example = tmpdir / "common" / "enterprise" / "example.inc"
            example.parent.mkdir(parents=True)
            example.touch()
            content = (
                'script_tag(name:"cvss_base", value:"4.0");\n'
                'script_tag(name:"summary", value:"Foo Bar...");\n'
                'script_dependencies("enterprise/example.inc");\n'
            )
            fake_context = self.create_file_plugin_context(
                nasl_file=path, file_content=content, root=tmpdir
            )
            plugin = CheckDependencies(fake_context)

            results = list(plugin.run())

            self.assertEqual(len(results), 0)

    def test_policy_warning(self):
        with self.create_directory() as tmpdir:
            path = tmpdir / "file.nasl"
            example = tmpdir / "common" / "Policy" / "example.inc"
            example.parent.mkdir(parents=True)
            example.touch()
            content = (
                'script_tag(name:"cvss_base", value:"4.0");\n'
                'script_tag(name:"summary", value:"Foo Bar...");\n'
                'script_dependencies("Policy/example.inc");\n'
            )
            fake_context = self.create_file_plugin_context(
                nasl_file=path, file_content=content, root=tmpdir
            )
            plugin = CheckDependencies(fake_context)

            results = list(plugin.run())

            self.assertEqual(len(results), 1)

            self.assertIsInstance(results[0], LinterWarning)
            self.assertEqual(
                "The script dependency Policy/example.inc is in a "
                "subdirectory, which might be misplaced.",
                results[0].message,
            )

    def test_error(self):
        with self.create_directory() as tmpdir:
            path = tmpdir / "file.nasl"
            example = tmpdir / "common" / "foo" / "example.inc"
            example.parent.mkdir(parents=True)
            example.touch()
            content = (
                'script_tag(name:"cvss_base", value:"4.0");\n'
                'script_tag(name:"summary", value:"Foo Bar...");\n'
                'script_dependencies("foo/example.inc");\n'
            )
            fake_context = self.create_file_plugin_context(
                nasl_file=path, file_content=content, root=tmpdir
            )
            plugin = CheckDependencies(fake_context)

            results = list(plugin.run())

            self.assertEqual(len(results), 1)

            self.assertIsInstance(results[0], LinterError)
            self.assertEqual(
                "The script dependency foo/example.inc is within a "
                "subdirectory, which is not allowed.",
                results[0].message,
            )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=fixme

import re
from enum import IntEnum
from pathlib import Path
from typing import Iterator, Union

from troubadix.helper import CURRENT_ENCODING, SpecialScriptTag
from troubadix.helper.helper import FEED_VERSIONS
from troubadix.helper.patterns import get_special_script_tag_pattern
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult

# See https://shorturl.at/jBGJT for a list of the category numbers.
class VTCategory(IntEnum):
    ACT_INIT = 0
    ACT_SCANNER = 1
    ACT_SETTINGS = 2
    ACT_GATHER_INFO = 3
    ACT_ATTACK = 4
    ACT_MIXED_ATTACK = 5
    ACT_DESTRUCTIVE_ATTACK = 6
    ACT_DENIAL = 7
    ACT_KILL_HOST = 8
    ACT_FLOOD = 9
    ACT_END = 10


class CategoryError(Exception):
    pass


def check_category(
    content: str, pattern: re.Pattern, script: str
) -> Union[LinterError, VTCategory]:
    """Check if the content contains a script category
    Arguments:
        content         the content to check

    Returns:
        LinterError     if no category found or category invalid
        VTCategory      else
    """
    match = pattern.search(content)

    if not match:
        raise CategoryError(
            f"{script}: Script category is missing or unsupported."
        )

    category_value = match.group("value")
    try:
        return VTCategory[category_value]
    except ValueError:
        raise CategoryError(
            f"{script}: Script category {category_value} is unsupported."
        ) from None


class CheckDependencyCategoryOrder(FileContentPlugin):
    name = "check_dependency_category_order"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """No VT N should depend on scripts that are in a category that
        normally would be executed after the category of VT M.
        e.g. a VT N within the ACT_GATHER_INFO category (3) is
        not allowed to depend on a VT M within the ACT_ATTACK category (4).
        See https://shorturl.at/jBGJT for a list of such category numbers.

        In addition it is not allowed for VTs to have a direct dependency
        to VTs from within the ACT_SCANNER category.
        """
        if not "script_dependencies(" in file_content:
            return

        category_pattern = get_special_script_tag_pattern(
            SpecialScriptTag.CATEGORY
        )

        try:
            category = check_category(
                content=file_content,
                pattern=category_pattern,
                script=nasl_file.name,
            )
        except CategoryError as e:
            yield LinterError(
                str(e),
                file=nasl_file,
                plugin=self.name,
            )
            return

        dependencies_pattern = get_special_script_tag_pattern(
            SpecialScriptTag.DEPENDENCIES
        )
        matches = dependencies_pattern.finditer(file_content)

        if not matches:
            return

        root = self.context.root

        for match in matches:
            if match:
                # Remove single and/or double quotes, spaces
                # and create a list by using the comma as a separator
                dependencies = re.sub(
                    r'[\'"\s]', "", match.group("value")
                ).split(",")

                for dep in dependencies:
                    # TODO: gsf/PCIDSS/PCI-DSS.nasl,
                    # gsf/PCIDSS/v2.0/PCI-DSS-2.0.nasl
                    # and GSHB/EL15/GSHB.nasl
                    # are using a variable which we currently can't handle.
                    if "+d+.nasl" in dep:
                        continue

                    dependency_path = None
                    for vers in FEED_VERSIONS:
                        if (root / vers / dep).exists():
                            dependency_path = root / vers / dep

                    if not dependency_path:
                        yield LinterError(
                            f"The script dependency {dep} could not "
                            "be found within the VTs.",
                            file=nasl_file,
                            plugin=self.name,
                        )
                    else:
                        dependency_content = dependency_path.read_text(
                            encoding=CURRENT_ENCODING
                        )

                        try:
                            dependency_category = check_category(
                                content=dependency_content,
                                pattern=category_pattern,
                                script=dependency_path.name,
                            )
                        except CategoryError as e:
                            yield LinterError(
                                str(e),
                                file=nasl_file,
                                plugin=self.name,
                            )

                        if category.value < dependency_category.value:
                            yield LinterError(
                                f"Script category {category.name}"
                                f"({category.value}) is lower than "
                                f"the category {dependency_category.name}"
                                f"({dependency_category.value}) of the "
                                f"dependency {dep}.",
                                file=nasl_file,
                                plugin=self.name,
                            )
                        # nb: Currently not sure about the
                        # host_alive_detection.nasl dependency so
                        # excluding them for now.
                        if (
                            dependency_category.name == "ACT_SCANNER"
                            and dep != "host_alive_detection.nasl"
                        ):
                            yield LinterError(
                                f"Script depends on {dep} which has the "
                                f"category {dependency_category.name}"
                                f"({dependency_category.value}), but no VT"
                                " is allowed to have a direct dependency "
                                "to VTs in this category.",
                                file=nasl_file,
                                plugin=self.name,
                            )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.cvss_format import CheckCVSSFormat

from . import PluginTestCase


class CheckCVSSFormatTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"cvss_base_vector", '
            'value:"AV:N/AC:L/Au:S/C:N/I:P/A:N");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVSSFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckCVSSFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_invalid_base(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"a12");\n'
            'script_tag(name:"cvss_base_vector", '
            'value:"AV:N/AC:L/Au:S/C:N/I:P/A:N");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVSSFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT has an invalid cvss_base value.",
            results[0].message,
        )

    def test_missing_base(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base_vector", '
            'value:"AV:N/AC:L/Au:S/C:N/I:P/A:N");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVSSFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT has an invalid cvss_base value.",
            results[0].message,
        )

    def test_invalid_vector(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"cvss_base_vector", '
            'value:"AV:N/AC:L/Au:S/C:N/I:P/A:");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVSSFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT has a missing or invalid cvss_base_vector value.",
            results[0].message,
        )

    def test_missing_vector(self):
        path = Path("some/file.nasl")
        content = 'script_tag(name:"cvss_base", value:"4.0");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVSSFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT has a missing or invalid cvss_base_vector value.",
            results[0].message,
        )

    def test_missing_cvss_base(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"");\n'
            'script_tag(name:"cvss_base_vector", '
            'value:"AV:N/AC:L/Au:S/C:N/I:P/A:N");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVSSFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertEqual(
            "VT has a missing cvss_base value.",
            results[0].message,
        )



# Copyright (C) 2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Argument parser for troubadix """

import sys
from argparse import ArgumentParser, Namespace
from multiprocessing import cpu_count
from pathlib import Path
from typing import Iterable

from pontos.terminal import Terminal


def directory_type(string: str) -> Path:
    directory_path = Path(string)
    if directory_path.exists() and not directory_path.is_dir():
        raise ValueError(f"{string} is not a directory.")
    return directory_path


def file_type(string: str) -> Path:
    file_path = Path(string)
    if file_path.exists() and not file_path.is_file():
        raise ValueError(f"{string} is not a file.")
    return file_path


def check_cpu_count(number: str) -> int:
    """Make sure this value is valid
    Default: use half of the available cores to not block the machine"""
    max_count = cpu_count()
    if not number:
        return max_count // 2
    number = int(number)
    if number > max_count:
        return max_count
    if number < 1:
        return max_count // 2
    return number


def parse_args(
    terminal: Terminal,
    args: Iterable[str] = None,
) -> Namespace:
    """Parsing args for troubadix

    Arguments:
        args        The program arguments passed by exec
    """

    parser = ArgumentParser(
        description="Greenbone NASL File Linter.",
    )

    parser.add_argument(
        "-V",
        "--version",
        dest="version",
        action="store_true",
        help=("Print the version of troubadix"),
    )

    parser.add_argument(
        "-f",
        "--full",
        action="store_true",
        help=(
            "Checking the complete VT directory and "
            "not only the added/changed scripts"
        ),
    )

    parser.add_argument(
        "--ignore-warnings",
        action="store_true",
        help="Set warnings to be quiet.",
    )

    parser.add_argument(
        "-r",
        "--root",
        type=directory_type,
        help="Root directory for the nasl files. Will be derived from the "
        "content if not set.",
    )

    what_group = parser.add_mutually_exclusive_group(required=False)

    what_group.add_argument(
        "-d",
        "--dirs",
        nargs="+",
        type=directory_type,
        help="List of directories that should be linted",
    )

    what_group.add_argument(
        "--files",
        nargs="+",
        type=file_type,
        help="List of files that should be linted",
    )

    what_group.add_argument(
        "--from-file",
        type=file_type,
        help=(
            "Pass a file that contains a List of files "
            "containing paths to files, that should be "
            "checked. Files should be separated by newline"
        ),
    )

    parser.add_argument(
        "--verbose",
        "-v",
        action="count",
        default=0,
        help=("-v verbose, -vv more verbose, -vvv debug"),
    )
    parser.add_argument(
        "--log-file",
        dest="log_file",
        type=file_type,
        help=("Log file path"),
    )

    parser.add_argument(
        "--log-file-statistic",
        dest="log_file_statistic",
        type=file_type,
        help=("Log file path for troubadix statistic"),
    )

    parser.add_argument(
        "--non-recursive",
        action="store_true",
        help=(
            "Don't run the script recursive. "
            'Only usable with "-f"/"--full" or "-d"/"--dirs"'
        ),
    )

    parser.add_argument(
        "--include-patterns",
        type=str,
        nargs="+",
        help=(
            "Allows to specify pattern(s) (glob) to "
            'limit the "--full"/"--dirs" run to specific file names. '
            'e.g. "gb_*.nasl", or "*some_vt*.nasl" or "some_dir/gb_*nasl". '
            'Only usable with "-f"/"--full" or "-d"/"--dirs".'
        ),
    )

    parser.add_argument(
        "--exclude-patterns",
        type=str,
        nargs="+",
        help=(
            "Allows to specify pattern(s) (glob) to "
            'exclude specific file names from the "--full"/"--dirs" run. '
            'e.g. "some_dir/*.nasl", "gb_*nasl", "*/anything.*'
            'Only usable with "-f"/"--full" or "-d"/"--dirs".'
        ),
    )

    tests_group = parser.add_mutually_exclusive_group(required=False)

    tests_group.add_argument(
        "--include-tests",
        type=str,
        nargs="+",
        dest="included_plugins",
        help=(
            "Allows to choose which tests should be run in this lint. "
            "Only the given tests will run. Valid as CamelCase and snake_case."
        ),
    )

    tests_group.add_argument(
        "--exclude-tests",
        type=str,
        nargs="+",
        dest="excluded_plugins",
        help=(
            "Allows to exclude tests from this lint. "
            "All tests excluding the given will run. "
            "Valid as CamelCase and snake_case."
        ),
    )

    parser.add_argument(
        "--fix",
        action="store_true",
        help="Try to fix specific issues during the linting.",
    )

    parser.add_argument(
        "-j",
        "--n-jobs",
        dest="n_jobs",
        default=cpu_count() // 2,
        type=check_cpu_count,
        help=(
            "Define number of jobs, that should run simultaneously. "
            "Default: %(default)s"
        ),
    )

    parser.add_argument(
        "--no-statistic",
        action="store_true",
        help="Don't print the statistic",
    )

    if not args:
        print("No arguments given.", file=sys.stderr)
        parser.print_help(sys.stdout)
        sys.exit(1)

    parsed_args = parser.parse_args(args=args)

    if (
        not parsed_args.full
        and not parsed_args.dirs
        and (parsed_args.include_patterns or parsed_args.exclude_patterns)
    ):
        terminal.warning(
            "The arguments '--include-patterns' and '--exclude-patterns' "
            "must be used with '-f/--full' or '-d'/'--dirs'"
        )
        sys.exit(1)

    if (
        not parsed_args.full
        and not parsed_args.dirs
        and parsed_args.non_recursive
    ):
        terminal.warning(
            "'Argument '--non-recursive' is only usable with "
            "'-f'/'--full' or '-d'/'--dirs'"
        )
        sys.exit(1)

    return parsed_args



# Copyright (C) 2021-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
from pathlib import Path
from subprocess import PIPE, Popen
from typing import AnyStr, List, Optional, Tuple, Union

# Script categories
SCRIPT_CATEGORIES = {
    "ACT_INIT": 0,
    "ACT_SCANNER": 1,
    "ACT_SETTINGS": 2,
    "ACT_GATHER_INFO": 3,
    "ACT_ATTACK": 4,
    "ACT_MIXED_ATTACK": 5,
    "ACT_DESTRUCTIVE_ATTACK": 6,
    "ACT_DENIAL": 7,
    "ACT_KILL_HOST": 8,
    "ACT_FLOOD": 9,
    "ACT_END": 10,
}


def is_ignore_file(
    file_name: Union[Path, str], ignore_files: Union[List[Path], List[str]]
) -> bool:
    for ignore_file in ignore_files:
        if str(ignore_file) in str(file_name):
            return True
    return False


ENTERPRISE_FOLDERS = (
    "enterprise",
    "gsf",
)

# Supported feed directories
FEED_VERSIONS = ["common", "21.04", "22.04", ""]


def is_enterprise_folder(folder: Union[Path, str]) -> bool:
    return str(folder) in ENTERPRISE_FOLDERS


def get_path_from_root(file_name: Path, root: Path):
    file_name = file_name.resolve()
    return file_name.relative_to(root.absolute())


# https://stackoverflow.com/questions/377017/test-if-executable-exists-in-python
def which(program: Union[Path, str]) -> Optional[Path]:
    def is_exe(fpath: Path):
        if isinstance(fpath, str):
            fpath = Path(fpath)
        return fpath.is_file() and os.access(fpath, os.X_OK)

    if isinstance(program, (Path, str)) and is_exe(program):
        return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = Path(path) / program
            if is_exe(exe_file):
                return exe_file

    return None


def subprocess_cmd(command: str, encoding="UTF-8") -> Tuple[AnyStr, AnyStr]:
    def any2str(str_input: AnyStr) -> str:
        if isinstance(str_input, bytes):
            return str_input.decode(encoding).strip()
        elif isinstance(str_input, str):
            return str_input.strip()
        return ""

    process = Popen(
        command,
        stdout=PIPE,
        shell=True,
        encoding=encoding,
    )
    proc_stdout, proc_stderr = process.communicate()

    return any2str(proc_stdout), any2str(proc_stderr)


def get_root(path: Path) -> Path:
    """Get the root directory of the VTs"""
    path = path.resolve().absolute()
    for parent in path.parents:
        if parent.name in ["", "nasl"]:
            return parent

    return path



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.overlong_script_tags import CheckOverlongScriptTags

from . import PluginTestCase


class CheckOverlongScriptTagsTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"summary", value:"Shorter than 1000");\n'
            'script_tag(name:"impact", value:"Shorter than 1000");\n'
            'script_tag(name:"affected", value:"Shorter than 1000");\n'
            'script_tag(name:"insight", value:"Shorter than 1000");\n'
            'script_tag(name:"vuldetect", value:"Shorter than 1000");\n'
            'script_tag(name:"solution", value:"Shorter than 1000");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckOverlongScriptTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckOverlongScriptTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_no_nasl_file(self):
        path = Path("some/file.inc")
        content = 'script_tag(name:"summary", value:"Shorter than 1000");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckOverlongScriptTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ignore_file(self):
        path = Path("monstra_cms_mult_vuln")
        content = (
            'script_tag(name:"summary", value:"Shorter than 1000");\n'
            'script_tag(name:"impact", value:"Shorter than 1000");\n'
            'script_tag(name:"affected", value:"Shorter than 1000");\n'
            'script_tag(name:"insight", value:"Shorter than 1000");\n'
            'script_tag(name:"vuldetect", value:"Shorter than 1000");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckOverlongScriptTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_one_wrong_entry(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"summary", value:"Shorter than 1000");\n'
            'script_tag(name:"impact", value:"Shorter than 1000");\n'
            'script_tag(name:"affected", value:"Shorter than 1000");\n'
            'script_tag(name:"insight", value:"Shorter than 1000");\n'
            'script_tag(name:"vuldetect", value:"Shorter than 1000");\n'
            'script_tag(name:"solution", value:"Loooooooooooooooooooooooooooooo'
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            'oooooooooooooooooooooooooooooooooooooooooooooooonger");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckOverlongScriptTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            results[0].message,
            "Tag solution is to long with 3053 characters. Max 3000",
        )

    def test_multiple_wrong_entries(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"summary", value:"Loooooooooooooooooooooooooooooo'
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            'oooooooooooooooooooooooooooooooooooooooooooooooonger");\n'
            'script_tag(name:"impact", value:"Shorter than 1000");\n'
            'script_tag(name:"affected", value:"Shorter than 1000");\n'
            'script_tag(name:"insight", value:"Shorter than 1000");\n'
            'script_tag(name:"vuldetect", value:"Shorter than 1000");\n'
            'script_tag(name:"solution", value:"Loooooooooooooooooooooooooooooo'
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            'oooooooooooooooooooooooooooooooooooooooooooooooonger");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckOverlongScriptTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            results[0].message,
            "Tag solution is to long with 3053 characters. Max 3000",
        )

        self.assertIsInstance(results[1], LinterError)
        self.assertEqual(
            results[1].message,
            "Tag summary is to long with 3053 characters. Max 3000",
        )

    def test_all_wrong_entries(self):
        path = Path("some/files.nasl")
        content = (
            'script_tag(name:"summary", value:"Loooooooooooooooooooooooooooooo'
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            'oooooooooooooooooooooooooooooooooooooooooooooooonger");\n'
            'script_tag(name:"impact", value:"Loooooooooooooooooooooooooooooo'
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            'oooooooooooooooooooooooooooooooooooooooooooooooonger");\n'
            'script_tag(name:"affected", value:"Loooooooooooooooooooooooooooooo'
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            'oooooooooooooooooooooooooooooooooooooooooooooooonger");\n'
            'script_tag(name:"insight", value:"Loooooooooooooooooooooooooooooo'
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            'oooooooooooooooooooooooooooooooooooooooooooooooonger");\n'
            'script_tag(name:"vuldetect", value:"Looooooooooooooooooooooooooooo'
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            'ooooooooooooooooooooooooooooooooooooooooooooo0ooonger");\n'
            'script_tag(name:"solution", value:"Loooooooooooooooooooooooooooooo'
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
            'oooooooooooooooooooooooooooooooooooooooooooooooonger");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckOverlongScriptTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 6)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            results[0].message,
            "Tag affected is to long with 3053 characters. Max 3000",
        )

        self.assertIsInstance(results[1], LinterError)
        self.assertEqual(
            results[1].message,
            "Tag impact is to long with 3053 characters. Max 3000",
        )

        self.assertIsInstance(results[5], LinterError)
        self.assertEqual(
            results[2].message,
            "Tag insight is to long with 3053 characters. Max 3000",
        )

        self.assertIsInstance(results[2], LinterError)
        self.assertEqual(
            results[3].message,
            "Tag solution is to long with 3053 characters. Max 3000",
        )

        self.assertIsInstance(results[3], LinterError)
        self.assertEqual(
            results[4].message,
            "Tag summary is to long with 3053 characters. Max 3000",
        )

        self.assertIsInstance(results[4], LinterError)
        self.assertEqual(
            results[5].message,
            "Tag vuldetect is to long with 3053 characters. Max 3000",
        )



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from itertools import chain
from typing import AnyStr, Iterator

from troubadix.helper import SpecialScriptTag, get_common_tag_patterns
from troubadix.helper.patterns import get_special_script_tag_pattern
from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckHttpLinksInTags(FilePlugin):
    name = "check_http_links_in_tags"

    def run(self) -> Iterator[LinterResult]:

        if self.context.nasl_file.suffix == ".inc":
            return chain()

        return chain(
            self.contains_http_link_in_tag(),
            self.contains_nvd_mitre_link_in_xref(),
        )

    def contains_http_link_in_tag(self) -> Iterator[LinterResult]:
        """Checks a given file if any of the
        script_tag(name:"(summary|impact|affected|insight|vuldetect|
        solution)", value:"")
        contains a http(s)://, ftp:(s)://, ftp. and/or www. link which
        should be moved to the following tag instead:

        script_xref(name:"URL", value:"");

        Args:
                nasl_file: The VT that is going to be checked
                file_content: The content of the file that is going to be
                            checked
        """

        file_content = self.context.file_content
        pattern = get_common_tag_patterns()
        tag_matches: Iterator[re.Match] = pattern.finditer(file_content)

        for tag_match in tag_matches:
            if tag_match:
                http_link_matches = re.finditer(
                    r".*((http|ftp)s?://|(www|\s+ftp)\.).*",
                    tag_match.group("value"),
                )
                if http_link_matches:
                    for http_link_match in http_link_matches:
                        if http_link_match:
                            if self.check_to_continue(http_link_match.group(0)):
                                continue

                            yield LinterError(
                                "One script_tag in the VT is using a HTTP "
                                "link/URL which should be moved to a separate "
                                '\'script_xref(name:"URL", value:"");\''
                                f" tag instead: '{tag_match.group(0)}'",
                                file=self.context.nasl_file,
                                plugin=self.name,
                            )

    def contains_nvd_mitre_link_in_xref(self) -> Iterator[LinterResult]:
        """
        Checks a given file if the script_xref(name:"URL", value:""); contains
        a link to an URL including any of this occurrence:

        - https://nvd.nist.gov/vuln/detail/CVE-

        - https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-

        Background: Both links are already covered by the script_cve_id() tag
        and the Link is just a redundant information.

        Args:
                nasl_file: The VT that is going to be checked
                file_content: The content of the file that is going to be
                                checked
        """

        file_content = self.context.file_content
        pattern = get_special_script_tag_pattern(SpecialScriptTag.XREF)
        tag_matches = pattern.finditer(file_content)

        for match in tag_matches:
            if match:
                if (
                    # fmt: off
                    "nvd.nist.gov/vuln/detail/CVE-" in match.group('ref')
                    or "cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-"
                    in match.group('ref')
                    # fmt: on
                ):
                    yield LinterError(
                        "The following script_xref is pointing "
                        "to Mitre/NVD which is already covered by the "
                        "script_cve_id. This is a redundant info and the "
                        f"script_xref needs to be removed: {match.group(0)}",
                        file=self.context.nasl_file,
                        plugin=self.name,
                    )

    @staticmethod
    def check_to_continue(http_link_match_group: AnyStr) -> bool:
        # When adding new entries to this list, please also add a testcase to
        # tests/plugins/test_http_links_in_tags.py -> test_http_link_in_tags_ok
        exclusions = [
            "The payloads try to open a connection to www.google.com",
            "The script attempts to connect to www.google.com",
            "to retrieve a web page from www.google.com",
            "Terms of use at https://www.verisign.com/rpa",
            "Subject: commonName=www.paypal.com",
            "example.com",
            "example.org",
            "www.exam",
            "sampling the resolution of a name (www.google.com)",
            "once with 'www.' and once without",
            "wget http://www.javaop.com/~ron/tmp/nc",
            "Ncat: Version 5.30BETA1 (http://nmap.org/ncat)",
            "as www.windowsupdate.com. (BZ#506016)",
            "located at http://sambarserver/session/pagecount.",
            "http://rest.modx.com",
            "ftp:// ",
            "ftp://'",
            "ftp://)",
            "ftp.c",
            "ftp.exe",
            "using special ftp://",
            "running ftp.",
            "ftp. The vulnerability",
            "'http://' protocol",
            "handle <a href='http://...'> properly",
            "Switch to git+https://",
            "wget https://compromised-domain.com/important-file",
            "the https:// scheme",
            "https://www.phishingtarget.com@evil.com",
            "distributions on ftp.proftpd.org have all been",
            "information from www.mutt.org:",
            "According to www.tcpdump.org:",
            "According to www.kde.org:",
            "From the www.info-zip.org site:",
            # pylint: disable=line-too-long
            " (www.isg.rhul.ac.uk) for discovering this flaw and Adam Langley and",
            "Sorry about having to reissue this one -- I pulled it from ftp.gnu.org not",
            # e.g.:
            # Since gedit supports opening files via 'http://' URLs
            "'http://'",
            "'https://'",
            "http://internal-host$1 is still insecure",
            "http:// ",
            "https:// ",
        ]

        return any(
            exclusion in http_link_match_group for exclusion in exclusions
        )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=fixme

import re
from typing import Iterator

from troubadix.helper import CURRENT_ENCODING, SpecialScriptTag
from troubadix.helper.helper import FEED_VERSIONS
from troubadix.helper.patterns import get_special_script_tag_pattern
from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckDeprecatedDependency(FilePlugin):
    name = "check_deprecated_dependency"

    def run(self) -> Iterator[LinterResult]:
        """No VT should depend on other VTs that are marked as deprecated via:

        script_tag(name:"deprecated", value:TRUE);
        exit(66);
        """

        if self.context.nasl_file.suffix == ".inc":
            return

        file_content = self.context.file_content

        if not "script_dependencies(" in file_content:
            return

        root = self.context.root

        dependencies_pattern = get_special_script_tag_pattern(
            SpecialScriptTag.DEPENDENCIES
        )
        matches = dependencies_pattern.finditer(file_content)
        if not matches:
            return

        deprecated_pattern = re.compile(
            r"^\s*exit\s*\(\s*66\s*\)\s*;|script_tag\s*\(\s*name\s*:\s*"
            r"(?P<quote>[\"'])deprecated(?P=quote)\s*,\s*value\s*:\s*TRUE"
            r"\s*\)\s*;",
            re.MULTILINE,
        )
        deprecated = deprecated_pattern.search(file_content)
        if deprecated:
            return

        for match in matches:
            if match:
                # Remove single and/or double quotes, spaces
                # and create a list by using the comma as a separator
                dependencies = re.sub(
                    r'[\'"\s]', "", match.group("value")
                ).split(",")

                for dep in dependencies:
                    # TODO: gsf/PCIDSS/PCI-DSS.nasl,
                    # gsf/PCIDSS/v2.0/PCI-DSS-2.0.nasl
                    # and GSHB/EL15/GSHB.nasl
                    # are using a variable which we currently can't handle.
                    if "+d+.nasl" in dep:
                        continue

                    dependency_path = None
                    for vers in FEED_VERSIONS:
                        if (root / vers / dep).exists():
                            dependency_path = root / vers / dep

                    if not dependency_path:
                        yield LinterError(
                            f"The script dependency {dep} could not "
                            "be found within the VTs.",
                            file=self.context.nasl_file,
                            plugin=self.name,
                        )
                    else:
                        dependency_content = dependency_path.read_text(
                            encoding=CURRENT_ENCODING
                        )

                        dependency_deprecated = deprecated_pattern.search(
                            dependency_content
                        )
                        if dependency_deprecated:
                            yield LinterError(
                                f"VT depends on {dep}, which is marked "
                                "as deprecated.",
                                file=self.context.nasl_file,
                                plugin=self.name,
                            )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=fixme

from enum import Enum
from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import (
    SpecialScriptTag,
    _get_special_script_tag_pattern,
)
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class ValidType(Enum):
    CHECKBOX = "checkbox"
    PASSWORD = "password"
    FILE = "file"
    RADIO = "radio"
    ENTRY = "entry"


class CheckScriptAddPreferenceType(FileContentPlugin):
    name = "check_script_add_preference_type"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This script checks the passed VT if it is using a
        script_add_preference not matching one of the following
        allowed strings passed to the 'type' function parameter:

            - checkbox
            - password
            - file
            - radio
            - entry

        Args:
            file: The VT that is going to be checked
        """

        if nasl_file.suffix == ".inc":
            return

        # don't need to check VTs not having a script_add_preference() call
        if "script_add_preference" not in file_content:
            return

        preferences_matches = _get_special_script_tag_pattern(
            name=SpecialScriptTag.ADD_PREFERENCE.value,
            value=r'type\s*:\s*(?P<quote>[\'"])(?P<type>[^\'"]+)'
            r"(?P=quote)\s*[^)]*",
        ).finditer(file_content)

        for preferences_match in preferences_matches:
            if preferences_match:

                pref_type = preferences_match.group("type")
                if pref_type not in [t.value for t in ValidType]:

                    # nb: This exists since years and it is currently
                    # unclear if we can change it so
                    # we're excluding it here for now.
                    if (
                        "ssh_authorization_init.nasl" in nasl_file.name
                        and pref_type == "sshlogin"
                    ):
                        continue

                    yield LinterError(
                        "VT is using an invalid or misspelled string "
                        f"({pref_type}) passed to the type parameter of "
                        "script_add_preference in "
                        f"'{preferences_match.group(0)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import get_xref_pattern
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckScriptXrefForm(FileContentPlugin):
    name = "check_script_xref_form"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """
        Checks for correct parameters for script_xref calls
        """
        if nasl_file.suffix == ".inc":
            return

        matches = re.finditer(r"script_xref\(.*\);", file_content)
        if matches:
            for match in matches:
                if match:
                    if not get_xref_pattern(name=r".*", value=r".*").match(
                        match.group(0)
                    ):
                        yield LinterError(
                            f"{match.group(0)}: does not conform to"
                            ' script_xref(name:"<name>", value:<value>);',
                            file=nasl_file,
                            plugin=self.name,
                        )
